#include "capture.h"

void PerformAudioCapture(float setvol, int time)
{
	const CLSID CLSID_MMDeviceEnumerator=__uuidof(MMDeviceEnumerator);
	const IID IID_IMMDeviceEnumerator=__uuidof(IMMDeviceEnumerator);
	const IID IID_IAudioClient = __uuidof(IAudioClient);
	IMMDeviceEnumerator* pEnumerator=NULL;
	IMMDevice* pDevice=NULL;
	LPWSTR pwszID = NULL;
	IAudioClient *pAudioClient = NULL;
	WAVEFORMATEX* pwfx=NULL; 
	const IID IID_IAudioCaptureClient = __uuidof(IAudioCaptureClient);
	IAudioCaptureClient *pCaptureClient = NULL;
	const IID IID_IChannelAudioVolume = __uuidof(IChannelAudioVolume);
	IChannelAudioVolume *pChannelAudioVolume = NULL;
	BYTE *pData=NULL;
	AudioSink* pMySink=NULL;
	PROPVARIANT varName;
	IPropertyStore *pProps = NULL;

	HRESULT hr=CoInitialize(0);

	try
	{
		hr=CoCreateInstance(CLSID_MMDeviceEnumerator, NULL, CLSCTX_ALL, IID_IMMDeviceEnumerator,(LPVOID*) &pEnumerator);

		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}
	
		hr=pEnumerator->GetDefaultAudioEndpoint(eCapture,eMultimedia,&pDevice);

		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}
		
		hr=pDevice->GetId(&pwszID);

		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}
	
		hr=pDevice->OpenPropertyStore(STGM_READ, &pProps);

		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}

		PropVariantInit(&varName);

		hr=pProps->GetValue(PKEY_Device_FriendlyName, &varName);
		
		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}

		printf("Input Device: \%S\ \n",varName.pwszVal);
		std::cout<<std::endl;

		hr=pDevice->Activate(IID_IAudioClient, CLSCTX_ALL,NULL, (void**)&pAudioClient);

		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}

		hr=pAudioClient->GetMixFormat(&pwfx);
		
		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}

		std::cout<<"Device format"<<std::endl;
		std::cout<<"Format is WAVE_FORMAT_EXTENSIBLE: "
			 <<((pwfx->wFormatTag)==WAVE_FORMAT_EXTENSIBLE)
			 <<" Channels: "
			 <<pwfx->nChannels
			 <<" Samples/sec: "
			 <<pwfx->nSamplesPerSec
			 <<" Bits/sample: "
			 <<pwfx->wBitsPerSample<<std::endl<<std::endl;

		PWAVEFORMATEXTENSIBLE pEx = reinterpret_cast<PWAVEFORMATEXTENSIBLE>(pwfx);
		if (IsEqualGUID(KSDATAFORMAT_SUBTYPE_IEEE_FLOAT, pEx->SubFormat)) 
		{
			pEx->SubFormat = KSDATAFORMAT_SUBTYPE_PCM;
			pwfx->nChannels=2;
			pwfx->nSamplesPerSec=44100;
			pwfx->wBitsPerSample = 16;
			pwfx->cbSize = 22;
			pEx->Samples.wValidBitsPerSample = pwfx->wBitsPerSample;
			pwfx->nBlockAlign = pwfx->nChannels * pwfx->wBitsPerSample / 8;
			pwfx->nAvgBytesPerSec = pwfx->nBlockAlign * pwfx->nSamplesPerSec;
			std::cout<<"External format set"<<std::endl;
			std::cout<<"Format is WAVE_FORMAT_EXTENSIBLE: "
					 <<((pwfx->wFormatTag)==WAVE_FORMAT_EXTENSIBLE)
					 <<" Channels: "
					 <<pwfx->nChannels
					 <<" Samples/sec: "
					 <<pwfx->nSamplesPerSec
					 <<" Bits/sample: "
					 <<pwfx->wBitsPerSample<<std::endl<<std::endl;
		}

		REFERENCE_TIME hnsRequestedDuration=REFTIMES_PER_SEC;


		hr=pAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED,0,hnsRequestedDuration,0,pwfx,NULL);


		UINT32 bufferFrameCount;
	
		hr=pAudioClient->GetBufferSize(&bufferFrameCount);

		hr=pAudioClient->GetService(IID_IAudioCaptureClient,(void**)&pCaptureClient);

		hr=pAudioClient->GetService(IID_IChannelAudioVolume,(void**)&pChannelAudioVolume);


		const float vol=setvol;
		hr=pChannelAudioVolume->SetAllVolumes(1,&vol,NULL);


		REFERENCE_TIME hnsActualDuration;

		hnsActualDuration=(double)((REFTIMES_PER_SEC*bufferFrameCount)/(pwfx->nSamplesPerSec));

		UINT32 packetLength=0;
		UINT32 numFramesAvailable;
		BOOL bDone=FALSE;
		DWORD flags;
		

		pMySink=new AudioSink((44100*(time+2)),(pwfx->wBitsPerSample)*(pwfx->nChannels)/8,bufferFrameCount);

		hr=pAudioClient->Start();

		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}

		if(hr==S_OK)
		{
			std::cout<<"Capture started."<<std::endl;
		}

		while (bDone == FALSE)
		{
			Sleep(hnsActualDuration/REFTIMES_PER_MILLISEC/2);

			hr=pCaptureClient->GetNextPacketSize(&packetLength);

			if(hr!=S_OK)
			{
				throw WinAPIerrors();
			}

			while (packetLength!=0)
			{
				hr = pCaptureClient->GetBuffer(&pData,&numFramesAvailable,&flags, NULL, NULL);

				if(hr!=S_OK)
				{
					throw WinAPIerrors();
				}

				pMySink->CopyData(pData,numFramesAvailable,&bDone,flags);

				hr=pCaptureClient->ReleaseBuffer(numFramesAvailable);

				if(hr!=S_OK)
				{
					throw WinAPIerrors();
				}

				hr=pCaptureClient->GetNextPacketSize(&packetLength);

				if(hr!=S_OK)
				{
					throw WinAPIerrors();
				}
			}
		}

		hr=pAudioClient->Stop(); 
		if(hr!=S_OK)
		{
			throw WinAPIerrors();
		}
	} //Try block end

	catch(WinAPIerrors)
	{
		ClearAll(pMySink, pwszID, pwfx, varName, pProps, pDevice, pEnumerator, pAudioClient, pCaptureClient, pChannelAudioVolume);
		throw WinAPIerrors(); //Tovbbdobs, azrt kell, hogy a kls hv is elkaphassa/tudjon rla.
	}

	catch(MemoryShortage)
	{
		ClearAll(pMySink, pwszID, pwfx, varName, pProps, pDevice, pEnumerator, pAudioClient, pCaptureClient, pChannelAudioVolume);
		throw MemoryShortage();	//Tovbbdobs, azrt kell, hogy a kls hv is elkaphassa/tudjon rla.
	}

	std::cout<<"Capture stopped."<<std::endl;

	//pMySink->WriteFile();
	pMySink->WriteFileInt(time);
	
	const float vol_end=1.0;
	hr=pChannelAudioVolume->SetAllVolumes(1,&vol_end,NULL);

	std::cout<<"Everything is done!"<<std::endl;
	std::cout<<"Process Data start."<<std::endl;

	ClearAll(pMySink, pwszID, pwfx, varName, pProps, pDevice, pEnumerator, pAudioClient, pCaptureClient, pChannelAudioVolume);
}

void ClearAll(AudioSink* pMySink, LPWSTR pwszID, WAVEFORMATEX* pwfx, PROPVARIANT varName, IPropertyStore* pProps, IMMDevice* pDevice, IMMDeviceEnumerator* pEnumerator, IAudioClient* pAudioClient, IAudioCaptureClient* pCaptureClient, IChannelAudioVolume* pChannelAudioVolume)
{
	delete pMySink;

	CoTaskMemFree(pwszID);
	CoTaskMemFree(pwfx);
	PropVariantClear(&varName);
    SAFE_RELEASE(pProps)
    SAFE_RELEASE(pDevice)
	SAFE_RELEASE(pEnumerator)
	SAFE_RELEASE(pAudioClient)
    SAFE_RELEASE(pCaptureClient)
	SAFE_RELEASE(pChannelAudioVolume)
}

