#include "fazis.h"

long double freq(const std::vector<long double>& rhside_vector, int sample_rate, unsigned long num_of_samples)
{
	unsigned long nn, power_n;

	
	unsigned long k=0;
	while (true)
	{
		nn=unsigned long(pow(2.0,double(k)));
		if (nn>(2*num_of_samples))
		{
			nn=unsigned long(pow(2.0,double(k-1)));
			break;
		}
		k++;
	}

	k=0;
	while (true)
	{
		power_n=unsigned long(pow(2.0,double(k)));
		if (power_n>(num_of_samples))
		{
			power_n=unsigned long(pow(2.0,double(k-1)));
			break;
		}
		k++;
	}

	long double *fft_vector = new long double[nn];
	if(!fft_vector) {
		std::cerr << "Not enough memory!" << std::endl;
		return 1;
		}

	unsigned long count_j= 1;
	fft_vector[0]=rhside_vector[0];
	for ( unsigned long i=1; i<nn  ; i++) {
			if (i % 2 == 0){
            fft_vector[i]=rhside_vector[count_j];
			count_j++;
			}
			else{
				fft_vector[i]=0.0;
			}
		}

	fft(fft_vector, power_n);
	
	long double *Magnitude = new long double[power_n/2];
	for(unsigned long k=0; k<(power_n/2); k++){
	Magnitude[k]=fftMagnitude(fft_vector,power_n,k);
	}

	long double Vmax;
	unsigned long imax;
	Vmax=Magnitude[0];
	for(unsigned long i=1;i<power_n/2;i++)
	{
		if(Vmax<Magnitude[i])
		{
			Vmax = Magnitude[i];
			imax = i;
		}
	}

	long double freq  = fftFrequency(power_n,imax,sample_rate);

	delete[] fft_vector;
	delete[] Magnitude;
	return freq;
}


//trapez integral fv
long double trapezoid_integrate(std::vector<long double> &x,  std::vector<long double> &y)
{
    if (x.size() != y.size())
    {
        std::cerr << "U and I must be the same size" << std::endl;
        exit(1);
    }
    long double sum = 0.0;
    for (unsigned int i = 1; i < x.size(); i++)
    {
        sum += (x[i] - x[i-1]) * (y[i] + y[i-1]);
    }
    return (sum * 0.5);
}

//fazistenyezo kiertekelo fv, es eredmeny kiiratas!
void fazis(std::vector<long double> &t, std::vector<long double> &U, std::vector<long double> &I, long double f, unsigned long N)
{
    const long double pi = 4 * atan(1.0);
    long double T=1/f;
    unsigned long k= unsigned long(floor(t[N-1]/T));
    long double k_T=k*T;
	
	unsigned int x=0;
    while (t[x] < k_T)
    {
        x++;
    }

    std::vector<long double> P_t;
    std::vector<long double> int_t;
    long double doublePt;
    for (unsigned int i = 0; i <= x; i++)
    {
        doublePt=U[i]*I[i];
        P_t.push_back(doublePt);
        int_t.push_back(t[i]);
    }

    long double C=1/k_T;
    long double P = C*trapezoid_integrate(int_t, P_t);

    long double U_cs = *max_element(U.begin(), U.end());
    long double I_cs = *max_element(I.begin(), I.end());

    long double cosfi=(P*2)/(U_cs*I_cs);
    long double fi_rad = acos(cosfi);
    long double fi=(fi_rad*180)/pi;
 
    std::cout << "Real power (P_real): " << P << " [W]" <<std::endl;
    std::cout << "Phase difference (fi): " << fi << " [degree]" <<std::endl;
    std::cout << "Phase factor (cos fi): " << cosfi << std::endl;
	
	P_t.clear();
    int_t.clear();
}


int ProcessData()
{
    std::vector<long double> U, I, t;
    long double f, temp_U, temp_I;
    int counter;
    int Fs = 44100;
    std::string data_file = "record.dat";

    std::ifstream data(data_file.c_str());
    if(!data)
    {
        std::cerr << "Error: " << data_file << " Can not be opened!" << std::endl;
        exit(1);
    }
	
    counter=0;
	//U es I beolvasas, szeparalas a ket oszlobol
    while( !data.eof() )
    {

        counter++;
        if (counter % 2 == 0)
        {
            data >> temp_U;
            if( !data )
            {
                break;
            }
            U.push_back(temp_U);
        }
        else
        {
            data >> temp_I;
            if( !data )
            {
                break;
            }
            I.push_back(temp_I);
        }
    }
    data.close();
    
	unsigned long N = U.size();
    long double Ts = 1 / long double(Fs);	
	
	//t vektor legyartasa, az utolso eleme a meresi idot megadja!
	for (unsigned long i = 0; i < N; i++)
    {
        t.push_back((i+1)*Ts);
    }

	f=freq(U, Fs, N);
	std::cout<<std::endl<<std::endl << "Signal frequency (f): " << f << " [Hz]" <<std::endl;
	fazis(t, U, I, f, N);

    t.clear();
    U.clear();
    I.clear();
    return 0;
}